/* --------------------------------------------------------
   ini.c
   Bodgy loading system for Windows-style INI files
   By BlackAura, May 2003
   -------------------------------------------------------- */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>
#include "ini.h"

#define IS_SPACE(c) ((c == ' ')||(c == '\t'))
#define IS_NEWLINE(c) ((c == '\r')||(c == '\n'))

// Strip leading spaces
static char *strip_leading(char *string)
{
	if (!string || (0 == strlen(string)))
		return NULL;

	while ((*string) && (IS_SPACE(*string)))
		string++;
	return string;
}

// Strip trailing spaces - MODIFIES ORIGINAL
static char *strip_trailing(char *string)
{
	int len;

	if(!string)
		return NULL;

	len = strlen(string);
	if(len == 0)
		return NULL;

	while( (len > 0) && IS_SPACE(string[len-1]) )
		string[--len] = '\0';

	return string;
}

// Read a single line, return -1 on failure
static int read_line(FILE *f, char *line)
{
	char *c;
	int len;
	
	if(feof(f))
	{
		*line = 0;
		return -1;
	}

	c = fgets(line, INI_BUFSIZE, f);

	if(c == NULL)
	{
		*line = 0;
		return -1;
	}

	len = strlen(line);
	while( (len > 0) && IS_NEWLINE(line[len-1]) )
		line[--len] = '\0';

	return len;
}

#ifdef _arch_dreamcast
#include <ctype.h>
#endif

// Are two strings equal?
static int strequal(char *s1, char *s2)
{
	while(tolower(*s1) == tolower(*s2))
	{
		if(*s1 == '\0')
			return 1;
		s1++;
		s2++;
	}
	return 0;
}

// Parse a single line into a key/data pair
static void parseline(char *line, char *key, char *data)
{
	int len = 0;

	line = strip_leading(line);
	strcpy(key, line);
	strcpy(data, "");

	while(*line)
	{
		char *ptr;

		if ( (*line == '=') )
		{
			key[len] = 0;
			key = strip_trailing(key);
			line = strip_leading(line+1);

			while(line && (*line == '='))
				line = strip_leading(line+1);

			if (line)
			{
				strcpy(data, line);
				if (NULL != (ptr = strrchr(data, ';')) &&
				NULL == strchr(ptr, '\"'))
				{
					*ptr = '\0';
				}
				strip_leading(data);
			}
			return;
		}
		else
		{
			line++;
			len++;
		}
	}
}

// Parse an INI and load it into a load of linked lists
static void ini_parse(FILE *f, ini_t *ini)
{
	char line[INI_BUFSIZE];
	char key[INI_BUFSIZE];
	char data[INI_BUFSIZE];
	char *lp;

	section_t *c_sect = NULL;
	key_t *c_key = NULL;

	while(read_line(f, line) > -1)
	{
		lp = strip_leading(line);

		if( (lp == NULL) || (lp[0]==0) || ((lp[0] == '/') && (lp[1] == '/')) || (lp[0] == '#') )
			continue;

		if( lp[0] == '[' )
		{
			lp++;
			lp[strlen(lp)-1] = 0;
			c_sect = ini->sections;
			while(c_sect)
			{
				if(strequal(c_sect->name, lp))
					break;
				c_sect = c_sect->next;
			}
			if(!c_sect)
			{
				c_sect = (section_t *)malloc(sizeof(section_t));
				memset(c_sect, 0, sizeof(section_t));
				strncpy(c_sect->name, lp, INI_BUFSIZE);
				c_sect->next = ini->sections;
				ini->sections = c_sect;
			}
		}
		else
		{
			if(c_sect)
			{
				parseline(lp, key, data);
				c_key = c_sect->key;
				while(c_key)
				{
					if(strequal(c_key->name, key))
						break;
					c_key = c_key->next;
				}
				if(!c_key)
				{
					c_key = (key_t *)malloc(sizeof(key_t));
					memset(c_key, 0, sizeof(key_t));
					strncpy(c_key->name, key, INI_BUFSIZE);
					c_key->next = c_sect->key;
					c_sect->key = c_key;
				}
				strncpy(c_key->data, data, INI_BUFSIZE);
			}
		}
	}
}

// Load an INI file from disc
ini_t *ini_load(char *filename)
{
	FILE *f;
	ini_t *retval;

	// Check and open file
	if(!filename)
		return NULL;
	f = fopen(filename, "rb");
	if(!f)
		return NULL;

	// Allocate INI struct
	retval = (ini_t *)malloc(sizeof(ini_t));
	if(!retval)
	{
		fclose(f);
		return NULL;
	}
	memset(retval, 0, sizeof(ini_t));

	// Parse the sucker
	ini_parse(f, retval);
	fclose(f);
	return retval;
}

// Free an INI structure and it's key list
void ini_free(ini_t *ini)
{
	if(ini)
	{
		while(ini->sections)
		{
			section_t *s = ini->sections->next;

			while(ini->sections->key)
			{
				key_t *k = ini->sections->key->next;
				free(ini->sections->key);
				ini->sections->key = k;
			}

			free(ini->sections);
			ini->sections = s;
		}
		free(ini);
	}
}

// Search for a specified key in a section
char *ini_search(ini_t *ini, char *section, char *key, char *def)
{
	section_t *s;
	key_t *k;

	if(!ini)
		return def;

	s = ini->sections;
	while(s)
	{
		if(strequal(section, s->name))
		{
			k = s->key;
			while(k)
			{
				if(strequal(key, k->name))
					return k->data;
				k = k->next;
			}
		}
		s = s->next;
	}

	return def;
}
