#include "shared.h"
#include "menu.h"
#include <kos.h>

/* Configurable options */
options_t dc_options =
{
	1,							/* Frameskip */
	487,						/* M68k clock speed */
	228							/* Z80 clock speed */
};

static int exec_frames;			/* Frames executed */
static int draw_frames;			/* Frames drawn */
// static char status_line[256];	/* Contents of status line */
// static int status_active = 0;	/* Is status line active */
static int rom_loaded = 0;		/* Has a ROM been loaded */

static void *screen_buffer;		/* Main display - Buffer */
static pvr_ptr_t gen_texptr;	/* Main display - Texture */
static pvr_poly_hdr_t gen_hdr;	/* Main display - Poly header */

/* Set up the ROM disk... */
extern uint8 romdisk[];
KOS_INIT_ROMDISK(romdisk);

#define TEX_WIDTH 512
#define TEX_HEIGHT 512

#define SRC_TOP		(0.0f)
#define SRC_BUM		(0.4375f)
#define SRC_LEFT	((float)(bitmap.viewport.x) / (float)(TEX_WIDTH))
#define SRC_RIGHT	((float)(bitmap.viewport.x + bitmap.viewport.w) / (float)(TEX_WIDTH))

static void dc_init()
{
	pvr_poly_cxt_t gen_cxt;

	/* Initialise the PowerVR */
	vid_set_mode(DM_640x480, PM_RGB565);
	pvr_init_defaults();

	/* Allocate and set up the main display texture */
	gen_texptr = pvr_mem_malloc(TEX_WIDTH*TEX_HEIGHT*2);
	pvr_poly_cxt_txr(&gen_cxt, PVR_LIST_OP_POLY, PVR_TXRFMT_NONTWIDDLED | PVR_TXRFMT_RGB565, TEX_WIDTH, TEX_HEIGHT, gen_texptr, PVR_FILTER_TRILINEAR1);
	pvr_poly_compile(&gen_hdr, &gen_cxt);

	/* Set up the drawing buffer */
	screen_buffer = memalign(32, TEX_WIDTH*TEX_HEIGHT*2);
    memset(&bitmap, 0, sizeof(t_bitmap));
    bitmap.width  = TEX_WIDTH;
    bitmap.height = TEX_HEIGHT;
    bitmap.depth  = 16;
    bitmap.granularity = 2;
    bitmap.pitch = (bitmap.width * bitmap.granularity);
    bitmap.data = screen_buffer;
    bitmap.viewport.w = 256;
    bitmap.viewport.h = 224;
    bitmap.viewport.x = 0x20;
    bitmap.viewport.y = 0x00;
    bitmap.remap = 1;

	/* Initialise the error handler */
	error_init();

	/* TODO - Intialise assertion handler */
}

void emu_kill()
{
	/* Don't bother if there's nothing loaded */
	if(!rom_loaded)
		return;

	/* Shut down the Genesis */
	system_shutdown();
	rom_loaded = 0;
}

void emu_load(char *filename)
{
	int r;

	/* Unload any previous ROM */
	if(rom_loaded)
		emu_kill();

	/* Load the ROM */
	r = load_rom(filename);
	assert_msg(r, "Unable to load ROM");

	/* Initialise the Genesis */
    system_init();
    system_reset();
	rom_loaded = 1;
}

static int update_input_player(int p)
{
	maple_device_t	*dev;
	cont_state_t	*state;

	/* Find a controller */
	dev = maple_enum_type(p, MAPLE_FUNC_CONTROLLER);
	if(!dev)
		return 0;

	/* Get controller state */
	state = (cont_state_t *)maple_dev_status(dev);
	if(!state)
		return 0;

	/* Exit combo - Both triggers + start */
	if((state->ltrig > 128) && (state->rtrig > 128) && (state->buttons & CONT_START))
		return 1;

	/* D-Pad up and down */
	if(state->buttons & CONT_DPAD_UP)
		input.pad[p] |= INPUT_UP;
	else if(state->buttons & CONT_DPAD_DOWN)
		input.pad[p] |= INPUT_DOWN;

	/* D-Pad left and right */
	if(state->buttons & CONT_DPAD_LEFT)
		input.pad[p] |= INPUT_LEFT;
	else if(state->buttons & CONT_DPAD_RIGHT)
		input.pad[p] |= INPUT_RIGHT;

	/* Buttons */
	if(state->buttons & CONT_X)
		input.pad[p] |= INPUT_A;
	if(state->buttons & CONT_A)
		input.pad[p] |= INPUT_B;
	if(state->buttons & CONT_B)
		input.pad[p] |= INPUT_C;
	if(state->buttons & CONT_START)
		input.pad[p] |= INPUT_START;
	
	/* Exit combo was not pressed */
	return 0;
}

static int update_input()
{
	int p1res;
	memset(&input, 0, sizeof(t_input));
	p1res = update_input_player(0);
	update_input_player(1);
	return p1res;
}

static __inline__ uint32 getTimer()
{
	uint32 sec, msec;
	uint32 time;
	timer_ms_gettime(&sec, &msec);
	time = (sec * 1000) + msec;
	return time;
}

static void emu_run()
{
	int skip_frame = 0;
	pvr_vertex_t vert;
	int still_going;
	uint32 last_time;
	uint32 this_time;

	int fps_count = 0;
	float fps_scale = 0.0f;

	still_going = 1;
	exec_frames = 0;
	draw_frames = 0;
	last_time = getTimer();

	while(still_going)
	{
		/* FPS counter stuff */
		exec_frames++;

		/* Run a frame of emulation */
		if(!system_frame(skip_frame > 0))
		{
			system_reset();
			skip_frame = 1;
		}

		/* Update the controllers */
		if(update_input())
			still_going = 0;

		/* Update screen */
		if(!skip_frame)
		{
			draw_frames++;

			/* Transfer the image buffer over */
			pvr_txr_load_dma(screen_buffer, gen_texptr, TEX_WIDTH*256*2, 1);

			/* Start drawing the scene */
			pvr_wait_ready();
			pvr_scene_begin();
			pvr_list_begin(PVR_LIST_OP_POLY);

			/* Start drawing the rectangle */
			pvr_prim(&gen_hdr, sizeof(gen_hdr));
			vert.argb = PVR_PACK_COLOR(1.0f, 1.0f, 1.0f, 1.0f);
			vert.oargb = 0;
			vert.flags = PVR_CMD_VERTEX;

			/* Upper left vertex */
			vert.x = 0.0f;
			vert.y = 16.0f;
			vert.z = 1.0f;
			vert.u = SRC_LEFT;
			vert.v = SRC_TOP;
			pvr_prim(&vert, sizeof(vert));

			/* Upper right vertex */
			vert.x = 639.0f;
			vert.y = 16.0f;
			vert.z = 1.0f;
			vert.u = SRC_RIGHT;
			vert.v = SRC_TOP;
			pvr_prim(&vert, sizeof(vert));

			/* Lower left vertex */
			vert.x = 0.0f;
			vert.y = 463.0f;
			vert.z = 1.0f;
			vert.u = SRC_LEFT;
			vert.v = SRC_BUM;
			pvr_prim(&vert, sizeof(vert));

			/* Lower right vertex */
			vert.flags = PVR_CMD_VERTEX_EOL;
			vert.x = 639.0f;
			vert.y = 463.0f;
			vert.z = 1.0f;
			vert.u = SRC_RIGHT;
			vert.v = SRC_BUM;
			pvr_prim(&vert, sizeof(vert));

			/* Draw the FPS counter */
			pvr_list_finish();
			pvr_list_begin(PVR_LIST_TR_POLY);
			dcmenu_printf(16, 16, "FPS: %i", fps_count);
			dcmenu_printf(16, 32, "SPD: %f", fps_scale);

			/* Finish drawing the scene */
			pvr_list_finish();
			pvr_scene_finish();
		}

		/* Decrement the frameskip counter */
		skip_frame--;
		if(skip_frame < 0)
			skip_frame = dc_options.frameskip;

		/* Update the FPS */
		this_time = getTimer();
		if(this_time - last_time > 1000)
		{
			/* Calculate the FPS and speed scale */
			fps_count = draw_frames;
			fps_scale = ((float)(exec_frames) / (60.0f)) * 100;

			/* Reset for next second */
			last_time = this_time;
			exec_frames = 0;
			draw_frames = 0;
		}
	}
}

/* This is a pretty simple main loop, isn't it? */
int main()
{
	dc_init();
	dcmenu_init();

	/* This should be in the ROM scanner... */
	while(1)
	{
		dcmenu_loop();
		if(rom_loaded)
			emu_run();
	}
}
