#include "shared.h"
#include "menu.h"
#include "main.h"
#include "ini.h"
#include <kos.h>
#include <imageload/imageload.h>
#include <jitter_table.h>

typedef struct iptr_s
{
	pvr_ptr_t texture;
	uint16 width, height;
} iptr_t;

static iptr_t *tex_warning;
static iptr_t *tex_credits;
static iptr_t *tex_font;
static iptr_t *tex_gamelist;
static iptr_t *tex_loading;
static iptr_t *tex_main;
static iptr_t *tex_options;
static iptr_t *tex_vmu;

typedef struct window_s
{
	int left;
	int right;
	int top;
	int bottom;
	int width;
	int height;
	int cwidth;
	int cheight;
	int cx;
	int cy;
	int red;
	int green;
	int blue;
	int alpha;
} window_t;

window_t win_rom;
window_t win_opt;
window_t win_vmu;

/* Small macro for packing float color values */
#define PVR_PACK_BYTE_COLOR(a, r, g, b) ( \
	( ((uint8)( a ) ) << 24 ) | \
	( ((uint8)( r ) ) << 16 ) | \
	( ((uint8)( g ) ) << 8 ) | \
	( ((uint8)( b ) ) << 0 ) )

/* --------------------------------------------------------
   Window setup bullshit
   -------------------------------------------------------- */
static void window_setup(ini_t *ini, char *section, window_t *win)
{
	win->left = atoi(ini_search(ini, section, "left", "72"));
	win->right = atoi(ini_search(ini, section, "right", "568"));
	win->top = atoi(ini_search(ini, section, "top", "72"));
	win->bottom = atoi(ini_search(ini, section, "bottom", "408"));
	win->red = atoi(ini_search(ini, section, "red", "51"));
	win->green = atoi(ini_search(ini, section, "green", "180"));
	win->blue = atoi(ini_search(ini, section, "blue", "51"));
	win->alpha = atoi(ini_search(ini, section, "alpha", "128"));
	win->width = win->right - win->left;
	win->height = win->bottom - win->top;
	win->cwidth = win->width / 16;
	win->cheight = win->height / 16;
	win->cx = (win->left + win->right) / 2;
	win->cy = (win->top + win->bottom) / 2;
}

/* --------------------------------------------------------
   Image handling stuff
   Not really part of the menu, but only the menu uses it
   -------------------------------------------------------- */
static iptr_t *load_image(char *filename, int alpha)
{
	int result;
	kos_img_t kos_img;
	pvr_ptr_t texture;
	iptr_t *image_struct;
	IMG_INFO loadinfo =
	{
		IMG_FILE_GUESS,
		IMG_ALPHA_NONE,
		0,
		IMG_DITHER_NONE,
		0, 0, 0
	};

	/* Check alpha mapping flag */
	if(alpha)
		loadinfo.alpha = IMG_ALPHA_FULL;

	/* Load the bitmap */
	result = img_load_file(filename, &loadinfo, &kos_img);

	/* Upload the texture to the PVR */
	texture = pvr_mem_malloc(kos_img.byte_count);
	pvr_txr_load_kimg(&kos_img, texture, 0);

	/* Create a little wrapper for the image */
	image_struct = (iptr_t *)malloc(sizeof(iptr_t));
	assert_msg(image_struct, "Out of memory allocating container");
	image_struct->texture = texture;
	image_struct->width = kos_img.w;
	image_struct->height = kos_img.h;

	/* Flush the kimg from main RAM - it's not needed anymore */
	kos_img_free(&kos_img, 1);

	/* Return the texture address */
	return image_struct;
}

static iptr_t *free_image(iptr_t *img)
{
	if(img)
	{
		if(img->texture)
			pvr_mem_free(img->texture);
		free(img);
	}
}

/* --------------------------------------------------------
   Basic drawing code
   -------------------------------------------------------- */
static void dcmenu_draw_start()
{
	/* Start a PVR frame */
	pvr_wait_ready();
	pvr_scene_begin();
	pvr_list_begin(PVR_LIST_OP_POLY);
}

static void dcmenu_draw_finish()
{
	/* Finish a frame */
	pvr_list_finish();
	pvr_scene_finish();
}

static void dcmenu_draw_trans()
{
	/* Jump to the transparent display list */
	pvr_list_finish();
	pvr_list_begin(PVR_LIST_TR_POLY);
}

/* Draw an image as the background */
static void dcmenu_draw_background(iptr_t *img, float fadeval)
{
	pvr_poly_cxt_t cxt;
	pvr_poly_hdr_t hdr;
	pvr_vertex_t vert;

	/* Create the polygon header */
	pvr_poly_cxt_txr(&cxt, PVR_LIST_OP_POLY, PVR_TXRFMT_RGB565, img->width, img->height, img->texture, PVR_FILTER_TRILINEAR1);
	pvr_poly_compile(&hdr, &cxt);

	/* Start drawing the rectangle */
	pvr_prim(&hdr, sizeof(hdr));
	vert.argb = PVR_PACK_COLOR(fadeval, fadeval, fadeval, fadeval);
	vert.oargb = 0;
	vert.flags = PVR_CMD_VERTEX;

	/* Upper left vertex */
	vert.x = 0.0f;
	vert.y = 0.0f;
	vert.z = 1.0f;
	vert.u = 0.0f;
	vert.v = 0.0f;
	pvr_prim(&vert, sizeof(vert));

	/* Upper right vertex */
	vert.x = 639.0f;
	vert.y = 0.0f;
	vert.z = 1.0f;
	vert.u = 1.0f;
	vert.v = 0.0f;
	pvr_prim(&vert, sizeof(vert));

	/* Lower left vertex */
	vert.x = 0.0f;
	vert.y = 479.0f;
	vert.z = 1.0f;
	vert.u = 0.0f;
	vert.v = 1.0f;
	pvr_prim(&vert, sizeof(vert));

	/* Lower right vertex */
	vert.flags = PVR_CMD_VERTEX_EOL;
	vert.x = 639.0f;
	vert.y = 479.0f;
	vert.z = 1.0f;
	vert.u = 1.0f;
	vert.v = 1.0f;
	pvr_prim(&vert, sizeof(vert));
}

/* Draw just a background image, nowt else */
static void dcmenu_draw_fullscreen(iptr_t *img, float fadeval)
{
	dcmenu_draw_start();
	dcmenu_draw_background(img, fadeval);
	dcmenu_draw_finish();
}

/* --------------------------------------------------------
   Controller code
   -------------------------------------------------------- */
static cont_state_t	*dcmenu_in_scan()
{
	maple_device_t	*dev;
	cont_state_t	*state;

	/* Find a controller */
	dev = maple_enum_type(0, MAPLE_FUNC_CONTROLLER);
	if(!dev)
		return NULL;

	/* Get controller state */
	state = (cont_state_t *)maple_dev_status(dev);
	if(!state)
		return NULL;

	/* Return the controller state */
	return state;
}

/* Wait for a button to be released */
static void dcmenu_in_waitrelease(int mask)
{
	cont_state_t *state;

	state = dcmenu_in_scan();
	while(state->buttons & mask)
		state = dcmenu_in_scan();
}

/* --------------------------------------------------------
   Intro/disclaimer display
   -------------------------------------------------------- */
static void draw_intro_fadein()
{
	int i;
	for(i = 0; i < 30; i++)
		dcmenu_draw_fullscreen(tex_warning, (float)i / 30.0f);		
	dcmenu_draw_fullscreen(tex_warning, 1.0f);
	dcmenu_draw_fullscreen(tex_warning, 1.0f);
}

static void draw_intro_fadeout()
{
	int i;
	for(i = 0; i < 240; i++)
		dcmenu_draw_fullscreen(tex_warning, 1.0f);
	for(i = 29; i > 0; i--)
		dcmenu_draw_fullscreen(tex_warning, (float)i / 30.0f);	
	dcmenu_draw_fullscreen(tex_warning, 0.0f);	
	dcmenu_draw_fullscreen(tex_warning, 0.0f);	
}

/* --------------------------------------------------------
   Font drawing stuff
   -------------------------------------------------------- */
static void dcmenu_draw_string(int x, int y, char *string)
{
	int ix1, ix2, iy1, iy2;
	float x1, x2, y1, y2;
	pvr_poly_cxt_t cxt;
	pvr_poly_hdr_t hdr;
	pvr_vertex_t vert;

	/* Create the transparent polygon header */
	pvr_poly_cxt_txr(&cxt, PVR_LIST_TR_POLY, PVR_TXRFMT_ARGB4444, tex_font->width, tex_font->height, tex_font->texture, PVR_FILTER_NONE);
	pvr_poly_compile(&hdr, &cxt);

	while(*string)
	{
		/* Calculate coordinates */
		ix1 = (*string % 16) * 16;
		iy1 = (*string / 16) * 16;
		ix2 = ix1 + 15;
		iy2 = iy1 + 15;
		x1 = (float)ix1 / 256.0f;
		x2 = (float)ix2 / 256.0f;
		y1 = (float)iy1 / 256.0f;
		y2 = (float)iy2 / 256.0f;

		/* Start drawing the rectangle */
		pvr_prim(&hdr, sizeof(hdr));
		vert.argb = PVR_PACK_COLOR(1.0f, 1.0f, 1.0f, 1.0f);
		vert.oargb = 0;
		vert.flags = PVR_CMD_VERTEX;

		/* Upper left vertex */
		vert.x = x + 0.0f;
		vert.y = y + 0.0f;
		vert.z = 1.0f;
		vert.u = x1;
		vert.v = y1;
		pvr_prim(&vert, sizeof(vert));

		/* Upper right vertex */
		vert.x = x + 15.0f;
		vert.y = y + 0.0f;
		vert.z = 1.0f;
		vert.u = x2;
		vert.v = y1;
		pvr_prim(&vert, sizeof(vert));

		/* Lower left vertex */
		vert.x = x + 0.0f;
		vert.y = y + 15.0f;
		vert.z = 1.0f;
		vert.u = x1;
		vert.v = y2;
		pvr_prim(&vert, sizeof(vert));

		/* Lower right vertex */
		vert.flags = PVR_CMD_VERTEX_EOL;
		vert.x = x + 15.0f;
		vert.y = y + 15.0f;
		vert.z = 1.0f;
		vert.u = x2;
		vert.v = y2;
		pvr_prim(&vert, sizeof(vert));

		/* Go to next character */
		x += 16;
		string++;
	}
}

static void dcmenu_draw_cstring(int x, int y, char *string)
{
	x -= strlen(string) * 8;
	dcmenu_draw_string(x, y, string);
}

void dcmenu_printf(int x, int y, char *string, ...)
{
	va_list argptr;
	char buffer[256];

	va_start(argptr, string);
	vsprintf(buffer, string, argptr);
	va_end(argptr);

	dcmenu_draw_string(x, y, buffer);
}

/* Draw a selection box in translucent mode */
static void dcmenu_draw_selbox(int yc, window_t *win)
{
	pvr_poly_cxt_t cxt;
	pvr_poly_hdr_t hdr;
	pvr_vertex_t vert;

	pvr_poly_cxt_col(&cxt, PVR_LIST_TR_POLY);
	pvr_poly_compile(&hdr, &cxt);

	/* Start drawing the rectangle */
	pvr_prim(&hdr, sizeof(hdr));
	vert.flags = PVR_CMD_VERTEX;
	vert.argb = PVR_PACK_BYTE_COLOR(win->alpha, win->red, win->green, win->blue);
	vert.oargb = 0;
	vert.z = 1.0f;

	/* Upper left vertex */
	vert.x = (float)win->left;
	vert.y = yc - 16.0f;
	pvr_prim(&vert, sizeof(vert));

	/* Upper right vertex */
	vert.x = (float)win->right;
	vert.y = yc - 16.0f;
	pvr_prim(&vert, sizeof(vert));

	/* Lower left vertex */
	vert.x = (float)win->left;
	vert.y = yc + 16.0f;
	pvr_prim(&vert, sizeof(vert));

	/* Lower right vertex */
	vert.flags = PVR_CMD_VERTEX_EOL;
	vert.x = (float)win->right;
	vert.y = yc + 16.0f;
	pvr_prim(&vert, sizeof(vert));
}

/* Draw a smaller selection box in translucent mode */
static void dcmenu_draw_selbox_small(int yc, window_t *win)
{
	pvr_poly_cxt_t cxt;
	pvr_poly_hdr_t hdr;
	pvr_vertex_t vert;

	pvr_poly_cxt_col(&cxt, PVR_LIST_TR_POLY);
	pvr_poly_compile(&hdr, &cxt);

	/* Start drawing the rectangle */
	pvr_prim(&hdr, sizeof(hdr));
	vert.flags = PVR_CMD_VERTEX;
	vert.argb = PVR_PACK_BYTE_COLOR(win->alpha, win->red, win->green, win->blue);
	vert.oargb = 0;
	vert.z = 1.0f;

	/* Upper left vertex */
	vert.x = (float)win->left;
	vert.y = yc - 8.0f;
	pvr_prim(&vert, sizeof(vert));

	/* Upper right vertex */
	vert.x = (float)win->right;
	vert.y = yc - 8.0f;
	pvr_prim(&vert, sizeof(vert));

	/* Lower left vertex */
	vert.x = (float)win->left;
	vert.y = yc + 8.0f;
	pvr_prim(&vert, sizeof(vert));

	/* Lower right vertex */
	vert.flags = PVR_CMD_VERTEX_EOL;
	vert.x = (float)win->right;
	vert.y = yc + 8.0f;
	pvr_prim(&vert, sizeof(vert));
}

/* --------------------------------------------------------
   ROM list
   -------------------------------------------------------- */
typedef char romfile_t[256];
static romfile_t romlist[1024];
static int romlist_init = 0;
static int romlist_count = 0;

static void romlist_clear()
{
	romlist_init = 0;
	romlist_count = 0;
	memset(romlist, 0, sizeof(romlist));
}

#define GAMES_DIRECTORY "/cd/games"

static void romlist_populate()
{
	int dir;
	dirent_t *ent;

	/* Reset the rom list */
	if(romlist_init)
		romlist_clear();

	/* Initialised, no ROMs found yet */
	romlist_init = 1;
	romlist_count = 0;

	/* Open the directory for listing */
	dir = fs_open(GAMES_DIRECTORY, O_RDONLY | O_DIR);
	if(dir == 0)
		return;

	/* Change to the games directory */
	fs_chdir(GAMES_DIRECTORY);

	/* Start listing */
	while((ent = fs_readdir(dir)) != NULL)
	{
		/* Skip directories */
		if((ent->size < 0))
			continue;

		/* Add entry to modlist */
		strncpy(romlist[romlist_count++], ent->name, sizeof(romfile_t));
	}
}

/* --------------------------------------------------------
   Menu main loop
   -------------------------------------------------------- */
#define MM_EXIT 0
#define MM_MAIN 1
#define MM_GAME 2
#define MM_OPTIONS 3
#define MM_VMU 4
#define MM_CREDITS 5

typedef struct menubranch_s
{
	int mask;
	int mode;
} menubranch_t;

static menubranch_t mbranch_main[] =
{
	{CONT_START, MM_EXIT},
	{CONT_A, MM_GAME},
	{CONT_B, MM_OPTIONS},
	{CONT_X, MM_VMU},
	{CONT_Y, MM_CREDITS}
};

/* Warning - This function contains horrible code.
   Those with a nervous disposition, you have been warned */
void dcmenu_loop()
{
	int menu_mode = MM_MAIN;
	cont_state_t *state;
	int i;
	int menu_selection = 0;
	int menu_offset = 0;
	int menu_countdown = 0;
	int menu_pressed = 0;
	int game_loading = 0;

	// Wait for the start button to be released, if we're coming from the emu
	state = dcmenu_in_scan();
	while(state->buttons & CONT_START)
		state = dcmenu_in_scan();

	while(menu_mode != MM_EXIT)
	{
		/* Update controller */
		state = dcmenu_in_scan();

		switch(menu_mode)
		{
			/* Main menu */
			case MM_MAIN:
				dcmenu_draw_fullscreen(tex_main, 1.0f);
				for(i = 0; i < sizeof(mbranch_main) / sizeof(menubranch_t); i++)
				{
					if(state->buttons & mbranch_main[i].mask)
					{
						dcmenu_in_waitrelease(mbranch_main[i].mask);
						menu_mode = mbranch_main[i].mode;
						menu_selection = 0;
						menu_offset = 0;
						menu_pressed = 0;
						menu_countdown = 0;
					}
				}
				break;

			/* ROM selection screen */
			case MM_GAME:
				if(state->buttons & CONT_B)
				{
					dcmenu_in_waitrelease(CONT_B);
					romlist_init = 0;
					menu_mode = MM_MAIN;
					break;
				}
				else if(romlist_init == 0)
				{
					romlist_populate();
				}
				else
				{
					int i;

					game_loading = 0;
					dcmenu_draw_start();
					dcmenu_draw_background(tex_gamelist, 1.0f);
					dcmenu_draw_trans();
					
					if(romlist_count > 0)
					{
						if(menu_countdown > 0)
							menu_countdown--;
						else if(state->buttons & CONT_A)
						{
							game_loading = 1;

							/* Fade the loading screen on */
							dcmenu_draw_finish();
							for(i = 0; i < 10; i++)
								dcmenu_draw_fullscreen(tex_loading, i / 10.0f);
							dcmenu_draw_fullscreen(tex_loading, 1.0f);
							dcmenu_draw_fullscreen(tex_loading, 1.0f);

							/* Load the ROM */
							emu_load(romlist[menu_selection + menu_offset]);
							
							/* Fade the loading screen off */
							for(i = 10; i > 0; i--)
								dcmenu_draw_fullscreen(tex_loading, i / 10.0f);
							dcmenu_draw_fullscreen(tex_loading, 0.0f);

							/* Jump to the emulator */
							return;
						}
						else
						{
							if(state->buttons & CONT_DPAD_DOWN)
							{
								menu_selection++;

								/* Scroll down */
								if(menu_selection >= win_rom.cheight)
								{
									menu_selection = win_rom.cheight - 1;
									menu_offset++;
								}

								/* Hit the bottom of the ROM list? */
								if(menu_selection + menu_offset >= romlist_count)
									menu_selection = romlist_count - menu_offset;

								/* Key repeater */
								if(menu_pressed == 0)
									menu_countdown = 20;
								else
									menu_countdown = 5;
								menu_pressed = 1;
							}
							else if(state->buttons & CONT_DPAD_UP)
							{
								menu_selection--;

								/* Scroll up */
								if(menu_selection < 0)
								{
									menu_selection = 0;
									menu_offset--;
								}

								/* Hit the top of the ROM list? */
								if(menu_offset < 0)
									menu_offset = 0;

								/* Key repeater */
								if(menu_pressed == 0)
									menu_countdown = 20;
								else
									menu_countdown = 5;
								menu_pressed = 1;
							}
							else if(state->buttons & CONT_DPAD_LEFT)
							{
								/* Scroll up a whole page */
								menu_offset -= win_rom.cheight;
								if(menu_offset < 0)
									menu_offset = 0;
								menu_selection = 0;

								/* Key repeater */
								if(menu_pressed == 0)
									menu_countdown = 20;
								else
									menu_countdown = 5;
								menu_pressed = 1;
							}
							else if(state->buttons & CONT_DPAD_RIGHT)
							{
								/* Scroll down a whole page */
								menu_offset += win_rom.cheight;
								if(menu_offset + win_rom.cheight > romlist_count)
									menu_offset = romlist_count - win_rom.cheight;
								if(menu_offset < 0)
									menu_offset = 0;
								menu_selection = 0;

								/* Key repeater */
								if(menu_pressed == 0)
									menu_countdown = 20;
								else
									menu_countdown = 5;
								menu_pressed = 1;
							}
							else
							{
								menu_pressed = 0;
								menu_countdown = 0;
							}
						}

						if(!game_loading)
						{
							/* Draw the list and selection box */
							dcmenu_draw_selbox_small( win_rom.top + (menu_selection * 16) + 8, &win_rom);
							for(i = 0; i < win_rom.cheight; i++)
							{
								if( (i + menu_offset) < romlist_count)
									dcmenu_draw_cstring(win_rom.cx, win_rom.top + (i*16), romlist[i + menu_offset]);
							}
						}
					}
					else
					{
						/* We got nothing - tell the user */
						dcmenu_draw_cstring(win_rom.cx, 232, "No games found");
					}

					/* Only finish up if we aren't drawing the loading screen */
					if(!game_loading)
						dcmenu_draw_finish();

					/* Pressed the refresh button yet? */
					if(state->buttons & CONT_Y)
					{
						dcmenu_in_waitrelease(CONT_Y);
						romlist_clear();
					}
				}
				break;

			/* Options screen */
			case MM_OPTIONS:
				if(state->buttons & CONT_B)
				{
					dcmenu_in_waitrelease(CONT_B);
					menu_mode = MM_MAIN;
					break;
				}
				else
				{
					dcmenu_draw_start();
					dcmenu_draw_background(tex_options, 1.0f);
					dcmenu_draw_trans();
					dcmenu_draw_selbox(win_opt.top + 16 + (menu_selection * 32), &win_opt);
					dcmenu_printf(168, win_opt.top + 8, "Frameskip: %d", dc_options.frameskip);
					dcmenu_printf(120, win_opt.top + 40, "M68000 Clock: %d", dc_options.m68_clock);
					dcmenu_printf(168, win_opt.top + 72, "Z80 Clock: %d", dc_options.z80_clock);
					dcmenu_printf(288, win_opt.top + 104, "Exit");
					dcmenu_draw_finish();

					if(menu_selection == 0)
					{
						/* Change frameskip */
						if(state->buttons & CONT_DPAD_LEFT)
						{
							if(dc_options.frameskip > 0)
								dc_options.frameskip--;
							dcmenu_in_waitrelease(CONT_DPAD_LEFT);
						}
						else if(state->buttons & CONT_DPAD_RIGHT)
						{
							if(dc_options.frameskip < 9)
								dc_options.frameskip++;
							dcmenu_in_waitrelease(CONT_DPAD_RIGHT);
						}
					}
					else if(menu_selection == 1)
					{
						/* Change M68000 clock */
						if(state->buttons & CONT_DPAD_LEFT)
						{
							if(dc_options.m68_clock > 50)
								dc_options.m68_clock -= 2;
						}
						else if(state->buttons & CONT_DPAD_RIGHT)
						{
							if(dc_options.m68_clock < 487)
								dc_options.m68_clock += 2;
						}
					}
					else if(menu_selection == 2)
					{
						/* Change Z80 Clock */
						if(state->buttons & CONT_DPAD_LEFT)
						{
							if(dc_options.z80_clock > 25)
								dc_options.z80_clock--;
						}
						else if(state->buttons & CONT_DPAD_RIGHT)
						{
							if(dc_options.z80_clock < 228)
								dc_options.z80_clock++;
						}
					}
					else if(menu_selection == 3)
					{
						/* Back to the main menu */
						if(state->buttons & CONT_A)
						{
							dcmenu_in_waitrelease(CONT_A);
							menu_mode = MM_MAIN;
							break;
						}
					}
					if(state->buttons & CONT_DPAD_UP)
					{
						/* Go up */
						menu_selection--;
						if(menu_selection < 0)
							menu_selection = 3;
						dcmenu_in_waitrelease(CONT_DPAD_UP);
					}
					else if(state->buttons & CONT_DPAD_DOWN)
					{
						/* Go down */
						menu_selection++;
						if(menu_selection > 3)
							menu_selection = 0;
						dcmenu_in_waitrelease(CONT_DPAD_DOWN);
					}
				}
				break;

			/* VMU options screen - TODO */
			case MM_VMU:
				if(state->buttons & CONT_B)
				{
					dcmenu_in_waitrelease(CONT_B);
					menu_mode = MM_MAIN;
					break;
				}
				else
				{
					dcmenu_draw_start();
					dcmenu_draw_background(tex_vmu, 1.0f);
					dcmenu_draw_trans();
					dcmenu_draw_selbox(win_vmu.top + 16, &win_opt);
					dcmenu_printf(288, win_vmu.top + 8, "Exit");
					dcmenu_draw_finish();
					if(state->buttons & CONT_A)
					{
						dcmenu_in_waitrelease(CONT_A);
						menu_mode = MM_MAIN;
						break;
					}
				}
				break;

			/* Credits screen */
			case MM_CREDITS:
				if(state->buttons & (CONT_A | CONT_B | CONT_X | CONT_Y | CONT_START))
				{
					dcmenu_in_waitrelease(CONT_A | CONT_B | CONT_X | CONT_Y | CONT_START);
					menu_mode = MM_MAIN;
					break;
				}
				else
				{
					dcmenu_draw_fullscreen(tex_credits, 1.0f);
				}
				break;
		}
	}
}

/* --------------------------------------------------------
	Initialisation stuff   
   -------------------------------------------------------- */
void dcmenu_init()
{
	ini_t *ini;

	/* Read the INI file */
	ini = ini_load("/cd/genplus/theme.ini");

	/* Display warning screen */
	tex_warning = load_image(ini_search(ini, "images", "warning", "/cd/genplus/warning.png"), 0);
	draw_intro_fadein();

	/* Load the textures */
	tex_credits = load_image(ini_search(ini, "images", "credits", "/cd/genplus/credits.png"), 0);
	tex_font = load_image(ini_search(ini, "images", "font", "/cd/genplus/font.png"), 1);
	tex_gamelist = load_image(ini_search(ini, "images", "gamelist", "/cd/genplus/gamelist.png"), 0);
	tex_loading = load_image(ini_search(ini, "images", "loading", "/cd/genplus/loading.png"), 0);
	tex_main = load_image(ini_search(ini, "images", "main", "/cd/genplus/main.png"), 0);
	tex_options = load_image(ini_search(ini, "images", "options", "/cd/genplus/options.png"), 0);
	tex_vmu = load_image(ini_search(ini, "images", "vmu", "/cd/genplus/vmu.png"), 0);

	/* Set up window metrics */
	window_setup(ini, "rom_menu", &win_rom);
	window_setup(ini, "vmu_menu", &win_vmu);
	window_setup(ini, "options_menu", &win_opt);

	/* Fade out the warning screen */
	draw_intro_fadeout();
}
